#include <linux/stddef.h>
#include <amlogic/aml_tv.h>
#include <amlogic/hdmitx/hdmi.h>

static struct hdmi_format_para fmt_para_1920x1080p60_16x9 = {
    .vic = HDMI_1920x1080p60_16x9,
    .name = "1920x1080p60hz",
    .pixel_repetition_factor = 0,
    .progress_mode = 1,
    .scrambler_en = 0,
    .tmds_clk_div40 = 0,
    .tmds_clk = 148500,
    .timing = {
        .pixel_freq = 148500,
        .h_freq = 67500,
        .v_freq = 60000,
        .vsync_polarity = 1,
        .hsync_polarity = 1,
        .h_active = 1920,
        .h_total = 2200,
        .h_blank = 280,
        .h_front = 88,
        .h_sync = 44,
        .h_back = 148,
        .v_active = 1080,
        .v_total = 1125,
        .v_blank = 45,
        .v_front = 4,
        .v_sync = 5,
        .v_back = 36,
        .v_sync_ln = 1,
    },
};

static struct hdmi_format_para fmt_para_1920x1080p50_16x9 = {
    .vic = HDMI_1920x1080p50_16x9,
    .name = "1920x1080p50hz",
    .pixel_repetition_factor = 0,
    .progress_mode = 1,
    .scrambler_en = 0,
    .tmds_clk_div40 = 0,
    .tmds_clk = 148500,
    .timing = {
        .pixel_freq = 148500,
        .h_freq = 56250,
        .v_freq = 50000,
        .vsync_polarity = 1,
        .hsync_polarity = 1,
        .h_active = 1920,
        .h_total = 2640,
        .h_blank = 720,
        .h_front = 528,
        .h_sync = 44,
        .h_back = 148,
        .v_active = 1080,
        .v_total = 1125,
        .v_blank = 45,
        .v_front = 4,
        .v_sync = 5,
        .v_back = 36,
        .v_sync_ln = 1,
    },
};

static struct hdmi_format_para fmt_para_1920x1080p24_16x9 = {
    .vic = HDMI_1920x1080p24_16x9,
    .name = "1920x1080p24hz",
    .pixel_repetition_factor = 0,
    .progress_mode = 1,
    .scrambler_en = 0,
    .tmds_clk_div40 = 0,
    .tmds_clk = 74250,
    .timing = {
        .pixel_freq = 74250,
        .h_freq = 27000,
        .v_freq = 24000,
        .vsync_polarity = 1,
        .hsync_polarity = 1,
        .h_active = 1920,
        .h_total = 2750,
        .h_blank = 830,
        .h_front = 638,
        .h_sync = 44,
        .h_back = 148,
        .v_active = 1080,
        .v_total = 1125,
        .v_blank = 45,
        .v_front = 4,
        .v_sync = 5,
        .v_back = 36,
        .v_sync_ln = 1,
    },
};

static struct hdmi_format_para fmt_para_3840x2160p30_16x9 = {
    .vic = HDMI_3840x2160p30_16x9,
    .name = "3840x2160p30hz",
    .pixel_repetition_factor = 0,
    .progress_mode = 1,
    .scrambler_en = 0,
    .tmds_clk_div40 = 0,
    .tmds_clk = 297000,
    .timing = {
        .pixel_freq = 297000,
        .h_freq = 67500,
        .v_freq = 30000,
        .vsync_polarity = 1,
        .hsync_polarity = 1,
        .h_active = 3840,
        .h_total = 4400,
        .h_blank = 560,
        .h_front = 176,
        .h_sync = 88,
        .h_back = 296,
        .v_active = 2160,
        .v_total = 2250,
        .v_blank = 90,
        .v_front = 8,
        .v_sync = 10,
        .v_back = 72,
        .v_sync_ln = 1,
    },
};

static struct hdmi_format_para fmt_para_3840x2160p60_16x9 = {
    .vic = HDMI_3840x2160p60_16x9,
    .name = "3840x2160p60hz",
    .pixel_repetition_factor = 0,
    .progress_mode = 1,
    .scrambler_en = 1,
    .tmds_clk_div40 = 1,
    .tmds_clk = 594000,
    .timing = {
        .pixel_freq = 594000,
        .h_freq = 135000,
        .v_freq = 60000,
        .vsync_polarity = 1,
        .hsync_polarity = 1,
        .h_active = 3840,
        .h_total = 4400,
        .h_blank = 560,
        .h_front = 176,
        .h_sync = 88,
        .h_back = 296,
        .v_active = 2160,
        .v_total = 2250,
        .v_blank = 90,
        .v_front = 8,
        .v_sync = 10,
        .v_back = 72,
        .v_sync_ln = 1,
    },
};

static struct hdmi_format_para fmt_para_3840x2160p50_16x9 = {
    .vic = HDMI_3840x2160p50_16x9,
    .name = "3840x2160p50hz",
    .pixel_repetition_factor = 0,
    .progress_mode = 1,
    .scrambler_en = 1,
    .tmds_clk_div40 = 1,
    .tmds_clk = 594000,
    .timing = {
        .pixel_freq = 594000,
        .h_freq = 112500,
        .v_freq = 50000,
        .vsync_polarity = 1,
        .hsync_polarity = 1,
        .h_active = 3840,
        .h_total = 5280,
        .h_blank = 1440,
        .h_front = 1056,
        .h_sync = 88,
        .h_back = 296,
        .v_active = 2160,
        .v_total = 2250,
        .v_blank = 90,
        .v_front = 8,
        .v_sync = 10,
        .v_back = 72,
        .v_sync_ln = 1,
    },
};

static struct hdmi_format_para fmt_para_3840x2160p25_16x9 = {
    .vic = HDMI_3840x2160p25_16x9,
    .name = "3840x2160p25hz",
    .pixel_repetition_factor = 0,
    .progress_mode = 1,
    .scrambler_en = 0,
    .tmds_clk_div40 = 0,
    .tmds_clk = 297000,
    .timing = {
        .pixel_freq = 297000,
        .h_freq = 56250,
        .v_freq = 25000,
        .vsync_polarity = 1,
        .hsync_polarity = 1,
        .h_active = 3840,
        .h_total = 5280,
        .h_blank = 1440,
        .h_front = 1056,
        .h_sync = 88,
        .h_back = 296,
        .v_active = 2160,
        .v_total = 2250,
        .v_blank = 90,
        .v_front = 8,
        .v_sync = 10,
        .v_back = 72,
        .v_sync_ln = 1,
    },
};

static struct hdmi_format_para fmt_para_3840x2160p24_16x9 = {
    .vic = HDMI_3840x2160p24_16x9,
    .name = "3840x2160p24hz",
    .pixel_repetition_factor = 0,
    .progress_mode = 1,
    .scrambler_en = 0,
    .tmds_clk_div40 = 0,
    .tmds_clk = 297000,
    .timing = {
        .pixel_freq = 297000,
        .h_freq = 54000,
        .v_freq = 24000,
        .vsync_polarity = 1,
        .hsync_polarity = 1,
        .h_active = 3840,
        .h_total = 5500,
        .h_blank = 1660,
        .h_front = 1276,
        .h_sync = 88,
        .h_back = 296,
        .v_active = 2160,
        .v_total = 2250,
        .v_blank = 90,
        .v_front = 8,
        .v_sync = 10,
        .v_back = 72,
        .v_sync_ln = 1,
    },
};

static struct hdmi_format_para fmt_para_4096x2160p24_256x135 = {
    .vic = HDMI_4096x2160p24_256x135,
    .name = "4096x2160p24hz",
    .pixel_repetition_factor = 0,
    .progress_mode = 1,
    .scrambler_en = 0,
    .tmds_clk_div40 = 0,
    .tmds_clk = 297000,
    .timing = {
        .pixel_freq = 297000,
        .h_freq = 54000,
        .v_freq = 24000,
        .vsync_polarity = 1,
        .hsync_polarity = 1,
        .h_active = 4096,
        .h_total = 5500,
        .h_blank = 1404,
        .h_front = 1020,
        .h_sync = 88,
        .h_back = 296,
        .v_active = 2160,
        .v_total = 2250,
        .v_blank = 90,
        .v_front = 8,
        .v_sync = 10,
        .v_back = 72,
        .v_sync_ln = 1,
    },
};

static struct hdmi_format_para fmt_para_1920x1080i60_16x9 = {
    .vic = HDMI_1920x1080i60_16x9,
    .name = "1920x1080i60hz",
    .pixel_repetition_factor = 0,
    .progress_mode = 0,
    .scrambler_en = 0,
    .tmds_clk_div40 = 0,
    .tmds_clk = 74250,
    .timing = {
        .pixel_freq = 74250,
        .h_freq = 33750,
        .v_freq = 60000,
        .vsync_polarity = 1,
        .hsync_polarity = 1,
        .h_active = 1920,
        .h_total = 2200,
        .h_blank = 280,
        .h_front = 88,
        .h_sync = 44,
        .h_back = 148,
        .v_active = 1080,
        .v_total = 1125,
        .v_blank = 45,
        .v_front = 2,
        .v_sync = 5,
        .v_back = 15,
        .v_sync_ln = 1,
    },
};

static struct hdmi_format_para fmt_para_1920x1080i50_16x9 = {
    .vic = HDMI_1920x1080i50_16x9,
    .name = "1920x1080i50hz",
    .pixel_repetition_factor = 0,
    .progress_mode = 0,
    .scrambler_en = 0,
    .tmds_clk_div40 = 0,
    .tmds_clk = 74250,
    .timing = {
        .pixel_freq = 74250,
        .h_freq = 28125,
        .v_freq = 50000,
        .vsync_polarity = 1,
        .hsync_polarity = 1,
        .h_active = 1920,
        .h_total = 2640,
        .h_blank = 720,
        .h_front = 528,
        .h_sync = 44,
        .h_back = 148,
        .v_active = 1080,
        .v_total = 1125,
        .v_blank = 45,
        .v_front = 2,
        .v_sync = 5,
        .v_back = 15,
        .v_sync_ln = 1,
    },
};

static struct hdmi_format_para fmt_para_1280x720p60_16x9 = {
    .vic = HDMI_1280x720p60_16x9,
    .name = "1280x720p60hz",
    .pixel_repetition_factor = 0,
    .progress_mode = 1,
    .scrambler_en = 0,
    .tmds_clk_div40 = 0,
    .tmds_clk = 74250,
    .timing = {
        .pixel_freq = 74250,
        .h_freq = 45000,
        .v_freq = 60000,
        .vsync_polarity = 1,
        .hsync_polarity = 1,
        .h_active = 1280,
        .h_total = 1650,
        .h_blank = 370,
        .h_front = 110,
        .h_sync = 40,
        .h_back = 220,
        .v_active = 720,
        .v_total = 750,
        .v_blank = 30,
        .v_front = 5,
        .v_sync = 5,
        .v_back = 20,
        .v_sync_ln = 1,
    },
};

static struct hdmi_format_para fmt_para_1280x720p50_16x9 = {
    .vic = HDMI_1280x720p50_16x9,
    .name = "1280x720p50hz",
    .pixel_repetition_factor = 0,
    .progress_mode = 1,
    .scrambler_en = 0,
    .tmds_clk_div40 = 0,
    .tmds_clk = 74250,
    .timing = {
        .pixel_freq = 74250,
        .h_freq = 37500,
        .v_freq = 50000,
        .vsync_polarity = 1,
        .hsync_polarity = 1,
        .h_active = 1280,
        .h_total = 1980,
        .h_blank = 700,
        .h_front = 440,
        .h_sync = 40,
        .h_back = 220,
        .v_active = 720,
        .v_total = 750,
        .v_blank = 30,
        .v_front = 5,
        .v_sync = 5,
        .v_back = 20,
        .v_sync_ln = 1,
    },
};

static struct hdmi_format_para fmt_para_720x480p60_16x9 = {
    .vic = HDMI_720x480p60_16x9,
    .name = "720x480p60hz",
    .pixel_repetition_factor = 0,
    .progress_mode = 1,
    .scrambler_en = 0,
    .tmds_clk_div40 = 0,
    .tmds_clk = 27000,
    .timing = {
        .pixel_freq = 27000,
        .h_freq = 31469,
        .v_freq = 59940,
        .vsync_polarity = 0,
        .hsync_polarity = 0,
        .h_active = 720,
        .h_total = 858,
        .h_blank = 138,
        .h_front = 16,
        .h_sync = 62,
        .h_back = 60,
        .v_active = 480,
        .v_total = 525,
        .v_blank = 45,
        .v_front = 9,
        .v_sync = 6,
        .v_back = 30,
        .v_sync_ln = 7,
    },
};

static struct hdmi_format_para fmt_para_720x480i60_16x9 = {
    .vic = HDMI_720x480i60_16x9,
    .name = "720x480i60hz",
    .pixel_repetition_factor = 1,
    .progress_mode = 0,
    .scrambler_en = 0,
    .tmds_clk_div40 = 0,
    .tmds_clk = 27000,
    .timing = {
        .pixel_freq = 27000,
        .h_freq = 15734,
        .v_freq = 59940,
        .vsync_polarity = 0,
        .hsync_polarity = 0,
        .h_active = 1440,
        .h_total = 1716,
        .h_blank = 276,
        .h_front = 38,
        .h_sync = 124,
        .h_back = 114,
        .v_active = 480,
        .v_total = 525,
        .v_blank = 45,
        .v_front = 4,
        .v_sync = 3,
        .v_back = 15,
        .v_sync_ln = 4,
    },
};

static struct hdmi_format_para fmt_para_720x576p50_16x9 = {
    .vic = HDMI_720x576p50_16x9,
    .name = "720x576p50hz",
    .pixel_repetition_factor = 0,
    .progress_mode = 1,
    .scrambler_en = 0,
    .tmds_clk_div40 = 0,
    .tmds_clk = 27000,
    .timing = {
        .pixel_freq = 27000,
        .h_freq = 31250,
        .v_freq = 50000,
        .vsync_polarity = 0,
        .hsync_polarity = 0,
        .h_active = 720,
        .h_total = 864,
        .h_blank = 144,
        .h_front = 12,
        .h_sync = 64,
        .h_back = 68,
        .v_active = 576,
        .v_total = 625,
        .v_blank = 49,
        .v_front = 5,
        .v_sync = 5,
        .v_back = 39,
        .v_sync_ln = 1,
    },
};

static struct hdmi_format_para fmt_para_720x576i50_16x9 = {
    .vic = HDMI_720x576i50_16x9,
    .name = "720x576i50hz",
    .pixel_repetition_factor = 1,
    .progress_mode = 0,
    .scrambler_en = 0,
    .tmds_clk_div40 = 0,
    .tmds_clk = 27000,
    .timing = {
        .pixel_freq = 27000,
        .h_freq = 15625,
        .v_freq = 50000,
        .vsync_polarity = 0,
        .hsync_polarity = 0,
        .h_active = 1440,
        .h_total = 1728,
        .h_blank = 288,
        .h_front = 24,
        .h_sync = 126,
        .h_back = 138,
        .v_active = 576,
        .v_total = 625,
        .v_blank = 49,
        .v_front = 2,
        .v_sync = 3,
        .v_back = 19,
        .v_sync_ln = 1,
    },
};

static struct hdmi_format_para *all_fmt_paras[] = {
    &fmt_para_3840x2160p60_16x9,
    &fmt_para_3840x2160p50_16x9,
    &fmt_para_3840x2160p30_16x9,
    &fmt_para_3840x2160p25_16x9,
    &fmt_para_3840x2160p24_16x9,
    &fmt_para_4096x2160p24_256x135,
    &fmt_para_1920x1080p50_16x9,
    &fmt_para_1920x1080p60_16x9,
    &fmt_para_1920x1080p24_16x9,
    &fmt_para_1920x1080i60_16x9,
    &fmt_para_1920x1080i50_16x9,
    &fmt_para_1280x720p60_16x9,
    &fmt_para_1280x720p50_16x9,
    &fmt_para_720x480p60_16x9,
    &fmt_para_720x480i60_16x9,
    &fmt_para_720x576p50_16x9,
    &fmt_para_720x576i50_16x9,
};

struct hdmi_format_para * hdmi_get_fmt_paras(HDMI_Video_Codes_t vic)
{
    int i;
    for(i = 0; i < sizeof(all_fmt_paras) / sizeof(struct hdmi_format_para*); i++) {
        if(vic == all_fmt_paras[i]->vic)
            return all_fmt_paras[i];
    }
    return NULL;
}
